/*@$***************************************************************************
* File Name	       : MjLog.cpp
*
*   COPYRIGHT Chen Mingjie 2001	  bloodchen@hotmail.com
*******************************************************************************/

#include <sstream>
#include <fstream>
#include <string>
#include "MjLog.h"
#include "time.h"
#ifdef WIN32
#include <windows.h>
#include <Winreg.h>
#endif
namespace MjTools
{
	void FileLogImpl::AddTimeStamp(std::string& LogStr)
	{
		time_t long_time;
		time( &long_time );                
		struct tm *pt = localtime( &long_time ); 
		if(pt)
		{
			std::ostringstream stream;
			stream<<"AT:"<<pt->tm_hour<<':'<<pt->tm_min<<':'<<pt->tm_sec<<','<<pt->tm_year+1900<<'/'<<pt->tm_mon+1<<'/'<<pt->tm_mday<<"-------->";
			LogStr=stream.str()+LogStr;
		}
	}
	bool FileLogImpl::AddLog(std::string& LogStr)
	{
		AddTimeStamp(LogStr);
		std::ofstream os(m_LogFileName.c_str(),std::ios::app);
		os<<LogStr;
		os<<std::endl;
		os.flush();
		os.close();
		return true;
	}
	void FileLogImpl::Clear()	   //clear file content
	{
		std::ofstream os(m_LogFileName.c_str(),std::ios::out);
		os.flush();
		os.close();
	}
#ifdef WIN32 //RegistryLogControl only valid in Windows system
	bool RegistryLogControl::CheckReg(const std::string& reg) //Check if required reg key exists
	{
		HKEY hKey;
		HKEY hKeyRoot;
		bool ret=true; 
		if(m_ControlReg!="") //if no control_reg_key provided,it means "no reg_key control"
		{
			int nPos=m_ControlReg.find('\\');
			if(nPos==-1)ret=false;
			else
			{
				std::string root=m_ControlReg.substr(0,nPos);
				std::string rest=m_ControlReg.substr(nPos+1);
				if(root=="HKEY_LOCAL_MACHINE")hKeyRoot=HKEY_LOCAL_MACHINE;
				if(root=="HKEY_CLASSES_ROOT")hKeyRoot=HKEY_CLASSES_ROOT;
				if(root=="HKEY_CURRENT_CONFIG")hKeyRoot=HKEY_CURRENT_CONFIG;
				if(root=="HKEY_CURRENT_USER")hKeyRoot=HKEY_CURRENT_USER;
				if(root=="HKEY_USERS")hKeyRoot=HKEY_USERS;
				if(root=="HKEY_PERFORMANCE_DATA")hKeyRoot=HKEY_PERFORMANCE_DATA;
				if(root=="HKEY_DYN_DATA")hKeyRoot=HKEY_DYN_DATA ;
				ret=(::RegOpenKey (hKeyRoot,rest.c_str(),&hKey)==ERROR_SUCCESS);
				::RegCloseKey(hKey);
			}
		}
	return ret;
	};
#endif
}
#ifdef _TEST_
int main()
{
	MjTools::CFileLog m_Log("test.log");
	std::string a="aaa";
	m_Log.Clear();
	m_Log.AddLog("Abc");
	m_Log.AddLog(a);
	MjTools::CFileLog m_Log1=m_Log;
	m_Log1.AddLog("From Log1");
#ifdef WIN32  //RegistryLogControl only valid in Windows system
	//construct a registry key controled log object. If the specified registry key is found,the log is enabled
	MjTools::CRegFileLog m_regLog("reglog.log","HKEY_LOCAL_MACHINE\\Software\\YourLogControlKey");
	m_regLog.AddLog("reglog");
	m_regLog.Pause();
	m_regLog.AddLog("reglog1");
	m_regLog.Resume();
	m_regLog.AddLog("reglog2");
#endif
	return 0;
}
#endif
