/*@$***************************************************************************
* File Name	       : MjLog.h
*
*   COPYRIGHT Chen Mingjie 2001	  bloodchen@hotmail.com
*******************************************************************************/
#ifndef _MjTools_CMjLog_
#define _MjTools_CMjLog_
namespace MjTools	 //Namespace defination for my tools
{
	class GeneralLogControl //Base class of any specific log control class,providing basic controls
	{
	protected:
		bool m_bEnabled;
	public:
		GeneralLogControl(const std::string ControlStr):m_bEnabled(true){};//log is enabled by default
		GeneralLogControl():m_bEnabled(true){};
		virtual ~GeneralLogControl(){};
		virtual void Pause(){m_bEnabled=false;}
		virtual void Resume(){m_bEnabled=true;}
		virtual bool CanAdd(std::string& LogStr){return m_bEnabled;}
	};
#ifdef WIN32	//RegistryLogControl only valid in Windows system
	class RegistryLogControl:public GeneralLogControl
	{
	protected:
		std::string m_ControlReg;
		bool CheckReg(const std::string& reg);
	public:
		RegistryLogControl(const std::string ControlStr)
			:m_ControlReg(ControlStr){m_bEnabled=CheckReg(ControlStr);}
		void Resume(){m_bEnabled=CheckReg(m_ControlReg);}
		virtual ~RegistryLogControl(){};
	};
#endif
	class FileLogImpl //class that implements file logging ability
	{
	protected:
		std::string m_LogFileName;
	public:
		FileLogImpl(const std::string FileName):m_LogFileName(FileName)
		{
			if(m_LogFileName=="")m_LogFileName="log.txt"; //default file name
		};
		virtual void AddTimeStamp(std::string& LogStr);	 //you may want to override this method to provide your time stamp style
		virtual ~FileLogImpl(){};
		virtual bool AddLog(std::string& LogStr);		 //you may want to override this method to provide your log style
		virtual void Clear();							 //clear log file content
	};
	template<class ImplT, class ControlT>
	class CMjLog
	{
	protected:
		ControlT m_Control;	  //log control class
		ImplT	 m_Impl;	  //log implementation class
	public:
		CMjLog(const std::string ImplStr="",const std::string ControlStr="")
		:m_Control(ControlStr),m_Impl(ImplStr){};
		virtual ~CMjLog(){};
		bool AddLog(const std::string& LogStr) //Add one piece of log message
		{
			return AddLog(LogStr.c_str());
		}
		bool AddLog(const char* pLogStr) //Add one piece of log message
		{
			std::string log=pLogStr;
			if(m_Control.CanAdd(log))
				return m_Impl.AddLog(log);
			else return false;
		}
		void Pause()   //Pause log process
		{
			m_Control.Pause();
		}
		void Resume() //Resume log process
		{
			m_Control.Resume();
		}
		void Clear()  //Clear log content
		{
			m_Impl.Clear();
		}

	};
	/*--------------CFileLog definition------------------*/
	typedef CMjLog<FileLogImpl,GeneralLogControl> CFileLog;
	/*------------CFileLog Usage-------------------------
		MjTools::CFileLog log("C:\\test.log"); //Construct a new logfile or open a existing log file
		log.Clear();				//Delete previous logs.
		log.AddLog("This is a test line"); //Add one log message
	---------------------------------------------------*/

	#ifdef WIN32 //RegistryLogControl only valid in Windows system
	/*--------------CRegFileLog definition------------------*/
	typedef CMjLog<FileLogImpl,RegistryLogControl> CRegFileLog;
	/*------------CRegFileLog Usage-------------------------
		MjTools::CRegFileLog log("C:\\test.log","HKEY_LOCAL_MACHINE\\Software\\YourLogControlKey"); //Construct a new logfile or open a existing log file
		log.Clear();				//Delete previous logs.
		log.AddLog("This is a test line"); //Add one log message
	---------------------------------------------------*/
	#endif
}
#endif
